from dataclasses import dataclass
from typing import override

from quark.core import Core, Data, Result
from quark.interface_types import InterfaceType, Other

from qiskit import QuantumCircuit

@dataclass
class CircuitProvider(Core):
    """This module provides a simple entangling circuit for testing purposes."""

    @override
    def preprocess(self, data: InterfaceType) -> Result:
        # For testing
        circuit = QuantumCircuit(2, 2)
        circuit.h(0)
        circuit.cx(0, 1)
        circuit.measure([0, 1], [0, 1])
        return Data(Other(circuit))

    @override
    def postprocess(self, result: Other[dict]) -> Result:
        self.counts = []
        for res in result.data["results"]:
            self.counts.append(res["data"]["counts"])
        return Data(Other(result))

    def get_metrics(self) -> dict:
        return {"counts": self.counts}



from dataclasses import dataclass
from typing import override
import os

from quark.core import Core, Data, Result
from quark.interface_types import InterfaceType, Other

from qiskit import QuantumCircuit
from mqss.qiskit_adapter import MQSSQiskitAdapter

lrz_token = os.getenv("LRZ_API_TOKEN")

@dataclass
class JobExecution(Core):
    """
    This module executes a quantum circuit on the MQSS backend using the Qiskit adapter.
    """

    @override
    def preprocess(self, data: Other[QuantumCircuit]) -> Result:
        circuit = data.data
        adapter = MQSSQiskitAdapter(token=lrz_token)
        backend = adapter.get_backend("QLM")
        self.job = backend.run(circuit, shots=1000, qasm3=False)
        return Data(Other(self.job))

    @override
    def postprocess(self, job: Other) -> Result:
        result_dict = self.job.result().to_dict()
        return Data(Other(result_dict))



from quark.plugin_manager import factory

from quark_plugin_mqss.circuit_provider import CircuitProvider
from quark_plugin_mqss.job_execution import JobExecution

def register() -> None:
    """
    Register all modules exposed to quark by this plugin.
    For each module, add a line of the form:
        factory.register("module_name", Module)

    The "module_name" will later be used to refer to the module in the configuration file.
    """
    factory.register("circuit_provider", CircuitProvider)
    factory.register("job_execution", JobExecution)
